defmodule FileStream.Error do
  defexception [:uri, :reason]

  @impl true
  def message(%{uri: uri, reason: reason}) do
    "Failed to stream #{uri}: #{reason}"
  end
end

defmodule FileStream.NotFoundError do
  defexception [:uri]

  @impl true
  def message(%{uri: uri}) do
    "Failed to stream #{uri}: resource not found."
  end
end

defmodule FileStream.PermissionError do
  defexception [:uri]

  @impl true
  def message(%{uri: uri}) do
    "Failed to stream #{uri}: permission not granted."
  end
end

defmodule FileStream.ConnectionTimeoutError do
  defexception [:uri]

  @impl true
  def message(%{uri: uri}) do
    "Failed to stream #{uri}: could not connect before timeout."
  end
end

defmodule FileStream.NXDomainError do
  defexception [:uri]

  @impl true
  def message(%{uri: uri}) do
    "Failed to stream #{uri}: the domain does not exist."
  end
end
